/*3:*/
#line 96 "analysis.w"


#ifndef ANALYSIS_HEADER_DEFINES
#define ANALYSIS_HEADER_DEFINES
#include <math.h>        

#include <iostream> 
#include <iomanip> 
#include <fstream> 
#include <exception> 
#include <stdexcept> 
#include <string> 
#include <vector> 
#include <map> 
using namespace std;

#include <ctype.h> 
#include <assert.h> 

#include "eggdata.h"
#include "timedate.h"
#include "statlib.h"

/*8:*/
#line 162 "analysis.w"

template<class Sample> class generic_egganalysis:public generic_eggsummary<Sample> {
public:

double*mean,*variance,*stouffer_z;
double*egg_mean,*egg_stdev;
Sample*egg_min_sample,*egg_max_sample;
int*egg_num_sample;
int*nsamples;

private:
void free_all_egg_analysis_data(void){
#define ReleaseTable(x) if ((x) != NULL) { delete (x); (x) =  NULL; }
ReleaseTable(mean);
ReleaseTable(variance);
ReleaseTable(stouffer_z);
ReleaseTable(nsamples);
#undef ReleaseTable
}

void free_per_egg_analysis_data(void){
#define ReleaseTable(x) if ((x) != NULL) { delete (x); (x) =  NULL; }
ReleaseTable(egg_mean);
ReleaseTable(egg_stdev);
ReleaseTable(egg_min_sample);
ReleaseTable(egg_max_sample);
ReleaseTable(egg_num_sample);
#undef ReleaseTable
}

public:

generic_egganalysis(){
mean= variance= stouffer_z= NULL;
nsamples= NULL;
egg_mean= egg_stdev= NULL;
egg_min_sample= egg_max_sample= NULL;
egg_num_sample= NULL;
}

void write_egg_statistics_CSV(string fileName);
void write_per_egg_statistics_CSV(string fileName);

void write_deviation_plot(string fileName,
string chartTitle= "",
int timeInterval= 1,int tickOffset= 0,
string x_axis_label= "",
double referenceProbability= 0.05,
systemtime referenceProbabilityStartTime= systemtime(0));

~generic_egganalysis(){
free_all_egg_analysis_data();
free_per_egg_analysis_data();
}

void compute_all_egg_statistics(void);

void compute_per_egg_statistics(void);
};


typedef generic_egganalysis<unsigned char> egganalysis;

/*:8*//*9:*/
#line 249 "analysis.w"

template<class Sample> void generic_egganalysis<Sample> ::compute_all_egg_statistics(void)
{
assert(generic_eggsummary<Sample> ::seconds_per_row);
int nitems= generic_eggsummary<Sample> ::seconds_of_data/generic_eggsummary<Sample> ::seconds_per_row;

free_all_egg_analysis_data();
mean= new double[nitems];
variance= new double[nitems];
stouffer_z= new double[nitems];
nsamples= new int[nitems];

int i;
for(i= 0;i<nitems;i++){
int j;
double rsum= 0,zsum= 0;
int n= 0;

for(j= 0;j<generic_eggsummary<Sample> ::eggs_reporting;j++){
if(isSampleValid(generic_eggsummary<Sample> ::get_egg_index(i,j))){
n++;
rsum+= generic_eggsummary<Sample> ::get_egg_index(i,j);
}
}
nsamples[i]= n;
mean[i]= rsum/n;
stouffer_z[i]= (mean[i]-(generic_eggsummary<Sample> ::trial_size/2))/(sqrt(generic_eggsummary<Sample> ::trial_size/4.0)/sqrt(static_cast<double> (n)));
if(isnan(stouffer_z[i])&&(i> 0)){
stouffer_z[i]= stouffer_z[i-1];
}
}
}

/*:9*//*10:*/
#line 299 "analysis.w"

template<class Sample> void generic_egganalysis<Sample> ::compute_per_egg_statistics(void){

/*11:*/
#line 313 "analysis.w"

free_per_egg_analysis_data();

egg_mean= new double[generic_eggsummary<Sample> ::eggs_reporting];
egg_stdev= new double[generic_eggsummary<Sample> ::eggs_reporting];
egg_min_sample= new Sample[generic_eggsummary<Sample> ::eggs_reporting];
egg_max_sample= new Sample[generic_eggsummary<Sample> ::eggs_reporting];
egg_num_sample= new int[generic_eggsummary<Sample> ::eggs_reporting];



int i;
int nitems= generic_eggsummary<Sample> ::seconds_of_data/generic_eggsummary<Sample> ::seconds_per_row;

for(i= 0;i<generic_eggsummary<Sample> ::eggs_reporting;i++){
egg_mean[i]= egg_stdev[i]= 0;
egg_min_sample[i]= generic_eggsummary<Sample> ::trial_size+1;
egg_max_sample[i]= 0;
egg_num_sample[i]= 0;
}

/*:11*/
#line 302 "analysis.w"
;
/*12:*/
#line 340 "analysis.w"

for(i= 0;i<generic_eggsummary<Sample> ::eggs_reporting;i++){
int j;

for(j= 0;j<nitems;j++){
Sample es= generic_eggsummary<Sample> ::get_egg_index(j,i);

if(isSampleValid(es)){
egg_num_sample[i]++;
egg_mean[i]+= es;
if(es<egg_min_sample[i]){
egg_min_sample[i]= es;
}
if(es> egg_max_sample[i]){
egg_max_sample[i]= es;
}
}
}
}


/*:12*/
#line 303 "analysis.w"
;
/*13:*/
#line 370 "analysis.w"

for(i= 0;i<generic_eggsummary<Sample> ::eggs_reporting;i++){
int j;

if(egg_num_sample[i]> 1){
egg_mean[i]/= egg_num_sample[i];
for(j= 0;j<nitems;j++){
Sample es= generic_eggsummary<Sample> ::get_egg_index(j,i);

if(isSampleValid(es)){
egg_stdev[i]+= (es-egg_mean[i])*(es-egg_mean[i]);
}
}
egg_stdev[i]= sqrt(egg_stdev[i]/(egg_num_sample[i]-1));
}
}


/*:13*/
#line 304 "analysis.w"
;
}

/*:10*//*16:*/
#line 413 "analysis.w"

template<class Sample> void generic_egganalysis<Sample> ::write_egg_statistics_CSV(string fileName)
{
FILE*of= fopen(fileName.c_str(),"w");
double ichisum= 0,chisum= 0;
int nitems= generic_eggsummary<Sample> ::seconds_of_data/generic_eggsummary<Sample> ::seconds_per_row;
int i,idof= 0;

/*14:*/
#line 393 "analysis.w"

if(mean==NULL){
compute_all_egg_statistics();
}

/*:14*/
#line 421 "analysis.w"
;
for(i= 0;i<nitems;i++){
double zsquare= stouffer_z[i]*stouffer_z[i];
double xe= chiSquareDistribution::x_from_p_k(0.99999,i+1);

chisum+= zsquare;
idof++;
fprintf(of,"24,%lu,%d,%.6f,%.6f,%.6f,%d,%.6f,%.6f\n",
generic_eggsummary<Sample> ::start_time.get_time()+(i*generic_eggsummary<Sample> ::seconds_per_row),
nsamples[i],
mean[i],
stouffer_z[i],
chisum,
idof,
xe,
(chisum-xe)/xe,
chiSquareDistribution::p_from_k_x(chisum,i+1)
);
}

fclose(of);
}


/*:16*//*17:*/
#line 463 "analysis.w"

template<class Sample> void generic_egganalysis<Sample> ::write_per_egg_statistics_CSV(string fileName)
{
ofstream of(fileName.c_str());

/*15:*/
#line 402 "analysis.w"

if(egg_mean==NULL){
compute_per_egg_statistics();
}

/*:15*/
#line 468 "analysis.w"
;
of<<setprecision(8);
for(int i= 0;i<generic_eggsummary<Sample> ::eggs_reporting;i++){
if(egg_num_sample[i]> 0){
of<<"25,"<<
generic_eggsummary<Sample> ::egg_number[i]<<","<<
egg_num_sample[i]<<","<<
egg_mean[i]<<","<<
((double)egg_max_sample[i])<<","<<
((double)egg_min_sample[i])<<","<<
egg_stdev[i]<<endl;
}
}
of.close();
}

/*:17*//*18:*/
#line 499 "analysis.w"

template<class Sample> void generic_egganalysis<Sample> ::write_deviation_plot(string fileName,
string chartTitle,
int timeInterval,int tickOffset,string x_axis_label,
double referenceProbability,
systemtime referenceProbabilityStartTime)
{
ofstream gp((fileName+".gp").c_str()),
dat((fileName+".dat").c_str());

/*14:*/
#line 393 "analysis.w"

if(mean==NULL){
compute_all_egg_statistics();
}

/*:14*/
#line 509 "analysis.w"
;
/*19:*/
#line 556 "analysis.w"

double ichisum= 0,chisum= 0;
int nitems= generic_eggsummary<Sample> ::seconds_of_data/generic_eggsummary<Sample> ::seconds_per_row;
int i,ref= 0,idof= 0;
double cxebase;
bool firstref= true;

for(i= 0;i<nitems;i++){
double zsquare= stouffer_z[i]*stouffer_z[i];
double xe= chiSquareDistribution::x_from_p_k(0.5,i+1);

chisum+= zsquare;
idof++;
dat<<i<<" "<<(chisum-xe);

if((generic_eggsummary<Sample> ::start_time.get_time()+(generic_eggsummary<Sample> ::seconds_per_row*i))>=referenceProbabilityStartTime.get_time()){
double xpRef= chiSquareDistribution::x_from_p_k(fabs(referenceProbability),ref+1);
if(firstref){
cxebase= chisum-xe;
firstref= false;
}
dat<<" "<<(((referenceProbability<0)?-1:1)*((chiSquareDistribution::x_from_p_k(0.5,ref+1)-xpRef)+cxebase));
ref++;
}
dat<<endl;
}


/*:19*/
#line 510 "analysis.w"
;



double final_p= chiSquareDistribution::p_from_k_x(idof,chisum);
gp<<"set term pbm small color"<<endl;
/*20:*/
#line 592 "analysis.w"

if(chartTitle==""){
chartTitle= generic_eggsummary<Sample> ::start_time.dateToString();
if((generic_eggsummary<Sample> ::start_time.get_time()!=generic_eggsummary<Sample> ::start_time.midnight())||
((generic_eggsummary<Sample> ::end_time.get_time()+1)!=(generic_eggsummary<Sample> ::start_time.get_time()+systemtime::SecondsPerDay))){
chartTitle+= " "+generic_eggsummary<Sample> ::start_time.timeToString();
chartTitle+= " -- ";
if(generic_eggsummary<Sample> ::start_time.midnight()!=generic_eggsummary<Sample> ::end_time.midnight()){
chartTitle+= generic_eggsummary<Sample> ::end_time.dateToString()+" ";
}
chartTitle+= generic_eggsummary<Sample> ::end_time.timeToString();
}
}
gp<<"set title \""<<chartTitle<<"\""<<endl;

/*:20*/
#line 516 "analysis.w"
;
gp<<"set ylabel \"Cumulative Deviation"<<endl;

/*21:*/
#line 617 "analysis.w"

if(x_axis_label!=" "){
if(x_axis_label==""){
switch(timeInterval){
case 1:
x_axis_label= "Seconds";
break;

case systemtime::SecondsPerMinute:
x_axis_label= "Minutes";
break;

case systemtime::SecondsPerHour:
x_axis_label= "Hours";
break;

case systemtime::SecondsPerDay:
x_axis_label= "Days";
break;

default:
x_axis_label= "Time Periods";
break;
}

x_axis_label+= " Elapsed";
}
gp<<"set xlabel \""<<x_axis_label<<"\""<<endl;
}


/*:21*/
#line 519 "analysis.w"
;

if(timeInterval!=1){
/*22:*/
#line 656 "analysis.w"

int ctime= generic_eggsummary<Sample> ::start_time.get_time()+tickOffset;
bool first= true;

gp<<"set xtics ( \\"<<endl;
for(i= 0;i<nitems;i++){
if((ctime%timeInterval)==0){
if(!first){
gp<<",\\"<<endl;
}
first= false;
gp<<"\""<<((i+tickOffset)/timeInterval)<<"\" "<<i;
}
ctime++;
}
gp<<"\\"<<endl<<"    )"<<endl;



/*:22*/
#line 522 "analysis.w"
;
}

gp<<"plot \""<<fileName<<".dat\" using 1:2 title \"Composite: "<<
generic_eggsummary<Sample> ::eggs_reporting<<" eggs; p = "<<setprecision(3)<<(1.0-final_p)<<
"\" with lines,\\"<<endl;
if(referenceProbability!=0){
gp<<"     \""<<fileName<<".dat\" using 1:3 title \"p="<<
referenceProbability<<"\" with lines,\\"<<endl;
}
gp<<"     0 title \"\""<<endl;

string command("gnuplot ");
command+= fileName+".gp | ppmtogif >"+fileName+".gif";
#ifdef DEV_PLOT_DEBUG
cout<<command<<endl;
#else
command+= " 2>/dev/null";
#endif
gp.close();
dat.close();
system(command.c_str());
#ifndef DEV_PLOT_DEBUG

remove((fileName+".gp").c_str());
remove((fileName+".dat").c_str());
#endif
}

/*:18*/
#line 119 "analysis.w"

#endif

/*:3*/
