/*3:*/
#line 133 "colour.w"


#ifndef COLOUR_HEADER_DEFINES
#define COLOUR_HEADER_DEFINES
#include <math.h>        

#include <iostream> 
#include <exception> 
#include <stdexcept> 
#include <string> 
#include <vector> 
#include <algorithm> 
using namespace std;

#include "bitmap.h"
#include <stdio.h>       
#include "gifout.h"
#include "graphics.h"
#include "pstampr.h"
#include "psrtext.h"

/*6:*/
#line 175 "colour.w"

class csColour{
public:
virtual string colourSystemName(void)= 0;

virtual void writeParameters(ostream&of){

of<<"Colour system: "<<colourSystemName()<<"\n";
}
};

/*:6*//*7:*/
#line 195 "colour.w"

class csSpectralColour:public csColour{
public:



virtual double getIntensity(double waveLength)= 0;




virtual bool isMonochromatic(void)= 0;
};

/*:7*//*8:*/
#line 216 "colour.w"

class csMonochromaticColour:public csSpectralColour{
private:
double wavelength;

public:
virtual string colourSystemName(void){
return"monochromatic spectral";
}

double getIntensity(double waveLength)
{
return(waveLength==wavelength)?1.0:0.0;
}

bool isMonochromatic(void){
return true;
}



csMonochromaticColour(void)
{
wavelength= 0.0;
}

csMonochromaticColour(double waveLength)
{
wavelength= waveLength;
}



void setWavelength(double waveLength)
{
wavelength= waveLength;
}

double getWavelength(void)
{
return wavelength;
}

void writeParameters(ostream&of){
csSpectralColour::writeParameters(of);
of<<"    Wavelength = "<<getWavelength()<<"\n";
}
};


/*:8*//*9:*/
#line 270 "colour.w"

class csSpectralBlack:public csSpectralColour{
public:
virtual string colourSystemName(void){
return"black spectral";
}

double getIntensity(double waveLength)
{
return 0.0;
}

bool isMonochromatic(void){
return false;
}
};

/*:9*//*10:*/
#line 292 "colour.w"

class csSpectralWhite:public csSpectralColour{
public:
virtual string colourSystemName(void){
return"white spectral";
}

double getIntensity(double waveLength)
{
return 1.0;
}

bool isMonochromatic(void){
return false;
}
};

/*:10*//*11:*/
#line 313 "colour.w"

class csBlackBody:public csSpectralColour{
private:

double temperature;

public:
double getIntensity(double waveLength)
{
return 0.0;
}

bool isMonochromatic(void){
return false;
}



csBlackBody(void)
{
temperature= 0.0;
}

csBlackBody(double temp)
{
temperature= temp;
}



void setTemperature(double temp)
{
temperature= temp;
}

double getTemperature(void)
{
return temperature;
}

double totalFlux(void)
{













return 5.67051e-8*
(temperature*temperature*temperature*temperature);
}

double flux(double wl)
{












return 3.74183e-16/((wl*wl*wl*wl*wl)*
(exp(1.4388e-2/(wl*temperature))-1));
}

void writeParameters(ostream&of){
csSpectralColour::writeParameters(of);
of<<"    Temperature = "<<getTemperature()<<" K\n";
}
};

/*:11*//*12:*/
#line 401 "colour.w"

class csDeviceColour:public csColour{
public:

/*13:*/
#line 419 "colour.w"

virtual void asRGB(double&r,double&g,double&b)= 0;
virtual void asCMYK(double&c,double&m,double&y,double&k)= 0;
virtual void asGreyScale(double&g)= 0;

/*:13*/
#line 405 "colour.w"

/*14:*/
#line 432 "colour.w"

virtual void asHSV(double&h,double&s,double&v);
virtual void asHLS(double&h,double&l,double&s);
virtual void asYIQ(double&y,double&i,double&q);
virtual void asYUV(double&y,double&u,double&v);
virtual void asSMPTE(double&y,double&Pb,double&Pr);
virtual void asCMY(double&c,double&m,double&y);

/*:14*/
#line 406 "colour.w"

/*15:*/
#line 446 "colour.w"

protected:
static double hlsval(double n1,double n2,double hue);

public:
static void hsv_rgb(double h,double s,double v,
double*r,double*g,double*b);
static void rgb_hsv(double r,double g,double b,
double*h,double*s,double*v);
static void rgb_hls(double r,double g,double b,
double*h,double*l,double*s);
static void hls_rgb(double h,double l,double s,
double*r,double*g,double*b);
static void rgb_yiq(double r,double g,double b,
double*y,double*i,double*q);
static void yiq_rgb(double y,double i,double q,
double*r,double*g,double*b);
static void rgb_yuv(double r,double g,double b,
double*y,double*u,double*v);
static void yuv_rgb(double y,double u,double v,
double*r,double*g,double*b);
static void rgb_smpte_204M(double r,double g,double b,
double*y,double*Pb,double*Pr);
static void smpte_204M_rgb(double y,double Pb,double Pr,
double*r,double*g,double*b);
static void rgb_cmy(double r,double g,double b,
double*c,double*m,double*y);
static void cmy_rgb(double c,double m,double y,
double*r,double*g,double*b);
static void cmy_cmyk(double c,double m,double y,
double*oc,double*om,double*oy,double*ok);
static void cmyk_cmy(double c,double m,double y,double k,
double*oc,double*om,double*oy);

/*:15*/
#line 407 "colour.w"

};

/*:12*//*33:*/
#line 1021 "colour.w"

class CIEColourSystem{
private:
string name;
double xRed,yRed,
xGreen,yGreen,
xBlue,yBlue,
xWhite,yWhite;

public:
CIEColourSystem(string c_name,
double c_xRed,double x_yRed,
double c_xGreen,double c_yGreen,
double c_xBlue,double c_yBlue,
double c_xWhite,double c_yWhite){
name= c_name;
xRed= c_xRed;
yRed= x_yRed;
xGreen= c_xGreen;
yGreen= c_yGreen;
xBlue= c_xBlue;
yBlue= c_yBlue;
xWhite= c_xWhite;
yWhite= c_yWhite;
}

void xyz_to_rgb(double xc,double yc,double zc,
double*r,double*g,double*b);

static bool inside_gamut(double r,double g,double b);

bool constrain_rgb(double*x,double*y,double*z,
double*r,double*g,double*b,
bool interpwp= false);

protected:
static double clamp(double v,double l,double h){
return(v<l)?l:((v> h)?h:v);
}
};

extern CIEColourSystem NTSCsystem,EBUsystem,SMPTEsystem,
HDTVsystem,CIEsystem;

/*:33*/
#line 154 "colour.w"

#endif

/*:3*/
